#include "kbbios.h"

char * const g_kbscanbuffer = (char *) KBBIOS_KBSBADDR;
char * const g_kbasciibuffer = (char *) KBBIOS_KBABADDR;
unsigned char g_kbscani; /* scan index (buffer position)*/
unsigned char g_kbascii; /* ascii input index (write position)*/
unsigned char g_kbascio; /* ascii output index (read  position)*/
Keyboard_state g_kbstate; /* keyboard device state */

#define KBBIOS_CLEAR_KBPORT kbport[0]=0xFFFFU


int 
kbbios_read_kbport( const unsigned int flags )
{
	unsigned int * const kbport = (unsigned int *) KBBASE;
	unsigned int portval;
	unsigned char scancode;

	/* read the keyboard port */
	portval = kbport[0];

	if ( portval & 0x0100U )
	{
		/* we got a scancode! */
		scancode = portval;

		/* clear port if flags[bit0] */
		if ( flags & KBFLAG_CLEAR_KBPORT )
			KBBIOS_CLEAR_KBPORT;

		if ( portval & 0x0200U)
			return KBBIOS_RC_BADPARITY;
		else
			return ((unsigned char) scancode);

	}
	else
        return KBBIOS_RC_NOCODE;
}

int
kbbios_compare_scancodes( const char key[], const char code[] )
{
	unsigned int j;
	char scancode, mastercode;
	unsigned endofsearch; /* loop control flag, terminate search? */
	unsigned keymatch;		/* loop status flag, do codes match? */

	/* validate function parameters */
	if ( key == 0 || code == 0 )
		return KBBIOS_RC_ILLEGAL_PARAMS;

	endofsearch = FALSE;
	keymatch = TRUE;
	for ( j=0; (!endofsearch) && j < KBBIOS_BUFLENGTH; ++j )
	{
		scancode = code[j];
		mastercode = key[j];

		if ( mastercode != scancode )
			keymatch = FALSE;

		/* check for end of string */
		if ( mastercode == '\0' || scancode == '\0' )
		{
			endofsearch = TRUE;
		}
	}	/* for ( j ... ) */

	if ( keymatch==TRUE )
		return TRUE;
	else
		return KBBIOS_RC_NOMATCH;
}

void 
kbbios_process_scancodes( void )
{
	int kbrc; /* keyboard-bios function return code */

	do {
		kbrc = kbbios_process_scancode();
	} while ( kbrc != KBBIOS_RC_NOCODE );
}

/*
 
   // converting scancodes to ASCII values is an iterative process...
   
	 // Procedural flow

	// 1) get scancode (kbbios_read_kbport)
	//
	// 2) Add scancode to "scanbuffer[]"

	// 3) Compare scanbuffer with key_table[]
	//    Can we match the scanbuffer contents with the key_table?
	//    Even if we get a match, is the match 'unique'?
	//
	// 4) If no match (or match and match!=unique), then
	//    return to host..."no new ASCII value"

	// 5) Otherwise, clear the scantable, 
	//    update the keyboard_device_state, 
	//    store the ASCII value (if applicable), 
	//    return to host
	//
	// *) If we get a 'end' code (0xF0), the state-machine will
	//    unconditionally reset 1 call later.
  //  

*/


int 
kbbios_process_scancode( void )
{
	unsigned i; /* loop variable */
	int kbrc; /* keyboard-bios function return code */
	unsigned char scancode;
	unsigned char scancode_type;
	unsigned char scancode_found; /* flag */

	unsigned int endofsearch;

	/* 1) kbrc = scancode */
	kbrc = kbbios_read_kbport( KBFLAG_CLEAR_KBPORT );

	if ( !KBBIOS_SUCCEED( kbrc ) )
		return kbrc;

	scancode = ((unsigned char) kbrc);

	/* 2) add scancode to scanbuffer, terminate scanbuffer with '\0' */
	g_kbscanbuffer[ g_kbscani ] = scancode;
	++g_kbscani;
	g_kbscanbuffer[ g_kbscani ] = '\0';
	
	/* 3) process the scantable */
	i = 0; 
	scancode_type = 0;
	scancode_found= FALSE;
	endofsearch = FALSE;
	while ( i < KEY_ENDOFTABLE && (!endofsearch) )
	{

		/* a) scan for breakcode match */
		kbrc = kbbios_compare_scancodes( kb_keys[i].pBreak, g_kbscanbuffer );
		scancode_type = 0; /* break code */
		
		/* b) scan for makecode match */
		if ( kbrc!=TRUE )
		{
			kbrc = kbbios_compare_scancodes( kb_keys[i].pMake, g_kbscanbuffer );
			scancode_type = 1; /* make code */
		}

		if ( kbrc == TRUE )
		{
			endofsearch = TRUE;
			scancode_found = TRUE;
			scancode = i;
		}
		++i;
	}	/* while ( i < ... ) */

	if ( scancode_found == TRUE )
		g_kbstate.flags.pending_bc = TRUE; 

	if ( g_kbstate.flags.pending_bc == TRUE )
	{
		g_kbscani = 0; /* reset scancode buffer pointer */
		g_kbscanbuffer[ g_kbscani ] = '\0'; /* clear scanbuffer */
		g_kbstate.flags.pending_bc = FALSE;
		if ( scancode_found==TRUE )
		{
			kbrc = kbbios_process_special( scancode, scancode_type );
			if ( kbrc == FALSE )
				kbrc = kbbios_process_ascii( scancode, scancode_type );
		}
	}

	if ( scancode == 0xF0U ) /* detected pending break code? */
		g_kbstate.flags.pending_bc = TRUE; 

	if ( scancode_found == FALSE )
		return KBBIOS_RC_UNKNOWN_SCANCODE;
	else
		return kbrc;
}

int 
kbbios_process_special( const int scancode_index, 
	const unsigned char scancode_type )
{
	unsigned char isMake = (scancode_type != 0);
	int isSpecial;
	unsigned int real_scancode;

	if ( scancode_index >= KEY_ENDOFTABLE )
		return KBBIOS_RC_ILLEGAL_PARAMS;

	real_scancode = kb_keys[ scancode_index ].id;
	isSpecial = FALSE;

	if ( real_scancode==KEY_LSHIFT )
	{
		g_kbstate.flags.shift_l = isMake;
		isSpecial=TRUE;
	}

	if ( real_scancode==KEY_RSHIFT )
	{
		g_kbstate.flags.shift_r = isMake;
		isSpecial=TRUE;
	}

	if ( real_scancode==KEY_CAPSLOCK )
	{
		/* toggle the capslock state */
		g_kbstate.flags.caps = g_kbstate.flags.caps ^ isMake;
		isSpecial=TRUE;
	}

	/* insert a '\r' before the '\n' of a carriage-return */
	if ( real_scancode==KEY_ENTER )
		kbbios_asciibuffer_add_key( '\r' );

	/*

	switch (scancode_id)
	{
		case KEY_LSHIFT :
			g_kbstate.flags.shift_l = isMake;
			break;

		case KEY_RSHIFT :
			g_kbstate.flags.shift_r = isMake;
			break;

		case KEY_LALT :
			g_kbstate.flags.alt_l = isMake;
			break;

		case KEY_RALT :
			g_kbstate.flags.alt_r = isMake;
			break;

		case KEY_LCTRL :
			g_kbstate.flags.ctrl_l = isMake;
			break;

		case KEY_RCTRL :
			g_kbstate.flags.ctrl_r = isMake;
			break;

		default :
			isSpecial = FALSE;
	}
	*/
	return isSpecial;
}


/* convert scancode to ASCII character, add to asciibuffer */
int 
kbbios_process_ascii( const int scancode_index, 
	const unsigned char scancode_type )
{
	unsigned char isMake = (scancode_type != 0);
	unsigned char ascii_value;
	int rc;
	unsigned int isShiftable; /* shift key affectable */
	unsigned int ShiftVal; /* shift value */

	if ( scancode_index >= KEY_ENDOFTABLE )
		return KBBIOS_RC_ILLEGAL_PARAMS;\

	ascii_value = kb_keys[ scancode_index ].ascii1;
	isShiftable = ( ascii_value >= 33 && ascii_value <= 126 );
	ShiftVal = (g_kbstate.flags.shift_l || g_kbstate.flags.shift_r) ^
		g_kbstate.flags.caps;

	if ( isShiftable && ShiftVal )
		ascii_value = kb_keys[ scancode_index ].ascii2;
	else
		ascii_value = kb_keys[ scancode_index ].ascii1;

	if ( ascii_value != 0 && scancode_type==1 ) /* make code? */
		rc = kbbios_asciibuffer_add_key( ascii_value );
	else
		rc = KBBIOS_RC_FAIL;


	return rc;
}

int
kbbios_asciibuffer_add_key( const unsigned char ascii_value )
{
	/* check full */
	if ( g_kbascii == (g_kbascio-1) )
		return KBBIOS_RC_BUFFER_FULL; /* ascii buffer full */

	/* add key to head of buffer */
	g_kbasciibuffer[ g_kbascii ] = ascii_value;
	++g_kbascii;
	if ( g_kbascii >= 16 )
		g_kbascii = 0;

	return KBBIOS_RC_OK;
}


int
kbbios_asciibuffer_get_key( void )
{
	unsigned char temp;
	/* check empty */
	if ( g_kbascii == g_kbascio )
		return KBBIOS_RC_BUFFER_EMPTY; /* ascii buffer empty */

	/* retrieve key from tail of buffer */
	temp = g_kbasciibuffer[ g_kbascio ];
	++g_kbascio;
	if ( g_kbascio >= 16 )
		g_kbascio = 0;

	return temp;
}


void
kbbios_init( void )
{
	g_kbscani = 0;
	g_kbscanbuffer[ g_kbscani ] = '\0'; /* clear scanbuffer */
	g_kbascii = 0;
	g_kbascio = 0;
	g_kbstate.word = 0;
	return;
}
