#ifndef KBBIOS_H_INCLUDED 
#define KBBIOS_H_INCLUDED 1

#define KBBIOS_RC_OK 0       /* generic success return code */
#define KBBIOS_RC_FAIL -1  /* generic failure code */
#define KBBIOS_RC_NOCODE -2  /* no key */
#define KBBIOS_RC_BADPARITY -3  /* parity on code was bad */
#define KBBIOS_RC_NOMATCH -4  /* scancode-compare, keys do not match */
#define KBBIOS_RC_ILLEGAL_PARAMS -5  /* illegal parameters */
#define KBBIOS_RC_BUFFER_FULL -6  /* ascii buffer full */
#define KBBIOS_RC_BUFFER_EMPTY -7  /* ascii buffer full */
#define KBBIOS_RC_UNKNOWN_SCANCODE -8  /* parsed scancode is unknown */

#define KBBASE (0xFF20U) /* KB-port I/O base address, 'peripheral[1]' */
#define KBBIOS_KBSBADDR (0x00C0U) /* scanbuffer address */
#define KBBIOS_KBABADDR (0x00D0U) /* asciibuffer address */

#define KBFLAG_CLEAR_KBPORT (0x0001U)

#define KBBIOS_SUCCEED( x )  ((x)>=0)

#define KBBIOS_BUFLENGTH (16U)

enum {
	KEY_A,
	KEY_B, 
	KEY_C, 
	KEY_D, 
	KEY_E, 
	KEY_F, 
	KEY_G, 
	KEY_H,
	KEY_I,
	KEY_J,
	KEY_K,
	KEY_L,
	KEY_M,
	KEY_N,
	KEY_O,
	KEY_P,
	KEY_Q,
	KEY_R,
	KEY_S,
	KEY_T,
	KEY_U,
	KEY_V,
	KEY_W,
	KEY_X,
	KEY_Y,
	KEY_Z,
	KEY_0,
	KEY_1,
	KEY_2,
	KEY_3,
	KEY_4,
	KEY_5,
	KEY_6,
	KEY_7,
	KEY_8,
	KEY_9,
	KEY_TICK, // '`'
	KEY_DASH, // '-'
	KEY_EQUAL, // '='
	KEY_BSLASH, // '\'
	KEY_FSLASH, // '/'
	KEY_BACKSPACE,
	KEY_SPACE, // ' '
	KEY_TAB,
	KEY_CAPSLOCK,
	KEY_LSHIFT,
	KEY_LCTRL,
	KEY_LWIN, // WIN95
	KEY_LALT,
	KEY_RSHIFT,
	KEY_RCTRL,
	KEY_RWIN, // WIN95
	KEY_RALT,
	KEY_APPS, // WIN95
	KEY_ENTER,
	KEY_ESC,
	KEY_F1,
	KEY_F2,
	KEY_F3,
	KEY_F4,
	KEY_F5,
	KEY_F6,
	KEY_F7,
	KEY_F8,
	KEY_F9,
	KEY_F10,
	KEY_F11,
	KEY_F12,
	KEY_LBRACKET, // '['
	KEY_INSERT,
	KEY_HOME,
	KEY_PAGEUP,
	KEY_DELETE,
	KEY_END,
	KEY_PAGEDOWN,
	KEY_UPARROW,
	KEY_LEFTARROW,
	KEY_DOWNARROW,
	KEY_RIGHTARROW,
	KEY_RBRACKET, // ']'
	KEY_SEMICOLON, // ';'
	KEY_APPOSTROPHE, // '''
	KEY_COMMA, // ','
	KEY_PERIOD, // '.'
	KEY_UNRECOGNIZED, // special code //
	KEY_ENDOFTABLE // special code //
} Keyboard_codes;


typedef struct {
	unsigned int id;
	char *pMake;  /* scancode list - 'make code' */
	char *pBreak; /* scancode list - 'break code' */
	char ascii1;  /* equivalent ascii code (base) */
	char ascii2;  /* equivalent ascii code (shift/ctrl) */
} Key_definition;

typedef struct {
		unsigned shift_l:1;	/* 'Left Shift' key is active */
		unsigned shift_r:1;	/* 'Right Shift' key is active */
		unsigned ctrl_l:1;	/* 'Left ctrl' key is active */
		unsigned ctrl_r:1;	/* 'Right ctrl' key is active */
		unsigned alt_l:1;	/* 'Left alt' key is active */
		unsigned alt_r:1;	/* 'Right alt' key is active */
		unsigned caps:1;	/* 'capslock' state (ON/oFF) */
		unsigned pending_bc:1;/* 'pending break code' */
		} struct_Keyboard_state_flags;
typedef union {
	unsigned int word;
	struct_Keyboard_state_flags flags;
} Keyboard_state;

#ifndef KBBIOS_TABLE_FILE
  extern const Key_definition kb_keys[];
#endif

#ifndef FALSE
 #define FALSE 0
#endif

#ifndef TRUE
 #define TRUE 1
#endif

/* FUNCTION PROTOTYPES */
int kbbios_read_kbport( const unsigned int flags );
int kbbios_compare_scancodes( const  char key[], const char code[] );
void kbbios_process_scancodes( void );
int kbbios_process_scancode( void );
int kbbios_process_special( const int scancode_index, 
	const unsigned char scancode_type );
int kbbios_process_ascii( const int scancode_index,
	const unsigned char scancode_type );
int kbbios_asciibuffer_add_key( const unsigned char ascii_value );
int kbbios_asciibuffer_get_key( void );
void kbbios_init( void );


#endif


