//////////////////////////////////////////////////////////////////////////////
//
// kbscanio.v
// ----------
//
//   XSOC I/O wrapper for kbscan.v(kbscan05.v)
//    allows XSOC to use the kbscan PS/2 scancode reader
//
//////////////////////////////////////////////////////////////////////////////

module kbscanio(

  // inputs
    clk, rst, 
    ctrl, sel, i_kbclk, i_kbdata,
    
  // tristate
    d,

  // outputs
    o_kbclk_en
   );

input			clk;		// global clock
input			rst;		// global async reset
input	[15:0]	ctrl;		// abstract control bus
input			sel;		// peripheral select
input	i_kbclk;	// from I/O pad
input	i_kbdata;	// from I/O pad

inout	[ 15:0]	d;			// LSB of on-chip data bus

output	o_kbclk_en;	// to tristate driver on I/O pad kbclk

tri		[15:0]	d;


// I/Os for kbscan (keyboard keypress processor)
wire       sc_i_kbclk;    // to I/O pad (io_kbclk)
wire       sc_i_kbdata;   // to I/O pad (i_kbdata)
wire       sc_i_byte_ack; // control, 'acknowledge' strobe 

wire [7:0] sc_o_byte;     // the scancode data
wire       sc_o_byte_rdy; // 
wire       sc_o_byte_err;
wire       sc_o_kbclk_en; // to I/O pad (io_kbclk)

wire [4:0]	addr;     // decoded I/O address 
wire		ud_t, ld_t, ud_ce, ld_ce;

ctrl_dec dec(.clk(clk), .rst(rst), .ctrl(ctrl), .sel(sel), .ud_t(ud_t),
			 .ld_t(ld_t), .ud_ce(ud_ce), .ld_ce(ld_ce), .addr(addr));

assign sc_i_kbclk    = i_kbclk;
assign sc_i_kbdata   = i_kbdata;
assign sc_i_byte_ack = ud_ce;

kbscan kbscan(
  // inputs
    .clk(clk), .rst(rst), 
    .i_kbclk(sc_i_kbclk), .i_kbdata(sc_i_kbdata), .i_byte_ack(sc_i_byte_ack),
    
  // outputs
    .o_byte(sc_o_byte), .o_byte_rdy(sc_o_byte_rdy), .o_byte_err(sc_o_byte_err),
    .o_kbclk_en(sc_o_kbclk_en)
   );

// assign module outputs

assign d[7:0] = ld_t ? 8'hZZ : sc_o_byte;
assign d[8]   = ud_t ? 1'bz  : sc_o_byte_rdy;
assign d[9]   = ud_t ? 1'bz  : sc_o_byte_err;
assign d[15:10]=ud_t ? 6'hZ  : 6'h0;
assign o_kbclk_en = sc_o_kbclk_en;

endmodule
