/* notes
   LCC quirks...
   don't mix unsigned and signed!
   Also append CONSTANTS with a 'U', do force unsigned logic.
   don't use "void *", completely confuses pointer address generation
   
   To move the cursor positon to start (left) of next line, the
   write_string() parser requires BOTH CR and LF! ('\n'+'\r'), this
   is an IBM-PC (MS-DOS) style text-format.

 */

#ifndef VBIOS_H_INCLUDED 
#define VBIOS_H_INCLUDED 1

#define VBIOS_PDD_ADDRESS (0x0080U)
#define VBIOS_VDD_ADDRESS (0x00A0U)

#define FALSE 0
#define TRUE 1

#define VBIOS_RC_OK 0  /* success code */

#define VBIOS_RC_FAIL -1  /* generic fail code */

#define VBIOS_RC_NOT_VISIBLE -2  /* co-ordinateo outside viewport */

#define VBIOS_RC_ILLEGAL_PARAMS -3  /* 1 or more parameters bad */

#define VBIOS_SCROLL_UP 1 /* for vbios_v_scroll_screen */
#define VBIOS_SCROLL_DOWN 2 /* for vbios_v_scroll_screen */

#define ASCII_SPACE 32 /* blank space ' ' */
#define ASCII_LF 10 /* CTRL-J '\n' */
#define ASCII_CR 13 /* CTRL-M '\r' */
#define ASCII_BACKSPACE 8 /* CTRL-H '\b' */

#define VBIOS_MODE_GRAPHICS1 (0x8000U)
#define VBIOS_MODE_GRAPHICS2 (0x8001U)

#define VBIOS_MODE_TEXT1     (0x0000U)
#define VBIOS_MODE_TEXT2     (0x0001U)


#define VBIOS_SUCCEED( x ) ( (x) == VBIOS_RC_OK )
#define VBIOS_GET_CHAR_SUCCEED( x ) ( (x) >= VBIOS_RC_OK )
#define VBIOS_MK_PTR( x ) (( unsigned char *)(x))
#define VBIOS_IS_MODE_TEXT( x ) (((x) & 0x8000U) == 0)
#define VBIOS_IS_MODE_GRAPHICS( x ) (((x) & 0x8000U))

typedef unsigned int word;

typedef struct 
{
	unsigned int w_gHeight; /* graphics screen height (#pixels) */
	unsigned int w_gWidth; /* graphics screen width (#pixels) */
	unsigned int w_gOffsetX; /* start of display Offset (#pixels) */
	unsigned int w_gOffsetY; /* start of display Offset (#pixels) */
	unsigned char c_fHeight; /* font character-cell height (#pixels) */
	unsigned char c_fWidth; /* font character-cell width (#pixels) */
	unsigned int c_tHeight; /* text screen height (#cells) */
	unsigned int c_tWidth; /* text screen width (#cells) */
	unsigned char *pGBase; /* pointer to (0,0) of graphics screen framebuffer */
	unsigned char *pTBase; /* pointer to (0,0) of text screen framebuffer */
	unsigned char *pFBase; /* pointer to (0,0) of font bitmap */
	unsigned int w_gPitch; /* pitch of graphics screen framebuffer (#bytes) */
	unsigned int w_tPitch; /* row pitch of text screen framebuffer (#bytes) */
	unsigned int w_mode;   /* display-mode selection */
} physical_display_descriptor;


typedef struct 
{
	/* viewport height (#cells), maximum-Y in viewport coordinate space */
	/* viewport width (#cells), maximum-X in viewport coordinate space */
	unsigned int c_tHeight;
	unsigned int c_tWidth; 

	/* X offset from base-coordinate (0,0), #cells */
	/* Y offset from base-coordinate (0,0), #cells */
	unsigned int c_tOffsetX; 
	unsigned int c_tOffsetY; 

	/* cursor position, X-coordinate (in viewport co-ordinate space) */
	/* cursor position, Y-coordinate (in viewport co-ordinate space) */
	unsigned int c_tCursorXpos; 
	unsigned int c_tCursorYpos; 
	/* ASCII character-code of the cursor blinker */
	unsigned char c_tCursorCode; 

	/* flag, 1=cursor is visible, 0=cursor invisible */
	unsigned int w_tCursorVisible; 

	/* pointer to physical-descriptor data-structure */
	physical_display_descriptor *pPDescriptor; 
} viewport_display_descriptor;


#ifndef __VBIOS_P_C_INCLUDED
	extern const physical_display_descriptor *g_pdd;
	extern const viewport_display_descriptor *g_vdd;
#endif

/****************************************************************************
 *
 * physical display routines
 *
 * Physical routines directly manipulate video controller RAM, or registers.
 *
 ****************************************************************************/

signed int vbios_set_font_address( const unsigned char font[] );
signed int vbios_set_framebuffer( const unsigned char framebuffer[] );
signed int vbios_set_textbuffer( const unsigned char textbuffer[] );
signed int vbios_set_gOffsetX( const unsigned int offset );
signed int vbios_set_gOffsetY( const unsigned int offset );
unsigned int vbios_get_gOffsetX( void );
unsigned int vbios_get_gOffsetY( void );


signed int vbios_p_get_char( const unsigned int tX, const unsigned int tY );

signed int vbios_p_put_char( const unsigned int tX, const unsigned int tY, 
					   const unsigned char ccode );

signed int vbios_p_draw_gchar( const unsigned int gX, const unsigned int gY, 
					   const unsigned char ccode );

signed int vbios_p_draw_gchar_fast( const unsigned int tX, const unsigned int gY, 
				const unsigned char ccode );

signed int vbios_p_init( void );

/****************************************************************************
 *
 * logical (viewport) display routines
 *
 * Viewport routines provide higher-level services to clients.  (For example,
 * writing a teletype-string to the screen (without automatic scrolling.)
 * The viewport routines internally map calls to the basic physical 
 * interface routiness.  
 *
 ****************************************************************************/

signed int vbios_v_get_char( const unsigned int tX, const unsigned int tY );

signed int vbios_v_put_char( const unsigned int tX, const unsigned int tY, 
					   const unsigned char ccode );

signed int vbios_v_write_char( const unsigned char ccode );

signed int vbios_v_write_string( const char cstring[] );

signed int vbios_v_move_cursor( const unsigned int tX, const unsigned int tY );

signed int vbios_v_scroll_screen( const unsigned int rowCount, 
				const unsigned int direction, const unsigned char fillcode );

signed int vbios_v_scroll_cursor( const int deltaX, const int deltaY );

signed int vbios_validate_physical_coordinates( const unsigned int tX, 
									  const unsigned int tY );

signed int vbios_validate_physical_graphics_coord( const unsigned int gX, 
										const unsigned int gY );

signed int vbios_validate_viewport_coordinates( const unsigned int tX, 
											   const unsigned int tY );

signed int vbios_v_init( void );

#endif


